package com.m2pool.chat.controller;

import com.m2pool.chat.service.ChatMessageService;
import com.m2pool.common.core.web.Result.AjaxResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/chat/message")
@Api(tags = "聊天消息相关接口")
public class ChatMessageController {

    @Autowired
    private ChatMessageService chatMessageService;

    /**
     * 查询7天前的历史记录
     * @return
     */
    @GetMapping("/find/history/message")
    @ApiOperation(value = "查询7天前的历史记录", notes = "根据ID查询7天前的历史记录")
    public AjaxResult findHistoryMessage(
            @ApiParam(value = "游标：最后一条消息id", required = true, example = "1")
            @RequestParam(defaultValue = "1") Long id,
            @ApiParam(value = "分页页码，默认为10条/页", example = "10")
            @RequestParam(required = false,defaultValue = "10")Integer pageNum,
            @ApiParam(value = "聊天室id", required = true, example = "1")
            @RequestParam Long roomId
            ) {
        return chatMessageService.findHistoryMessage(id,pageNum,roomId);
    }

    @GetMapping("/find/recently/message")
    @ApiOperation(value = "查询七天内的记录", notes = "根据ID查询七天内的记录")
    public AjaxResult findRecentlyMessage(
            @ApiParam(value = "游标：最后一条消息id", required = true, example = "1")
            @RequestParam(defaultValue = "1") Long id,
            @ApiParam(value = "分页页码，默认为10条/页", example = "10")
            @RequestParam(required = false,defaultValue = "10")Integer pageNum,
            @ApiParam(value = "聊天室id", required = true, example = "1")
            @RequestParam Long roomId
    ) {
        return chatMessageService.findRecentlyMessage(id,pageNum,roomId);
    }

    @GetMapping("/delete/message")
    @ApiOperation(value = "消息撤回或删除")
    public AjaxResult deleteMessage(
            @ApiParam(value = "消息id", required = true, example = "1")
            @RequestParam(defaultValue = "1") Long id,
            @ApiParam(value = "消息类型 0 文本 1 图片", example = "0")
            @RequestParam(required = false,defaultValue = "0")Integer type
    ) {
        return chatMessageService.deleteMessage(id,type);
    }

    @GetMapping("/read/message")
    @ApiOperation(value = "聊天室消息改已读")
    public AjaxResult readMessage(
            @ApiParam(value = "聊天室id", required = true, example = "1")
            @RequestParam Long roomId) {
        return null;
    }




}