package com.m2pool.chat.interceptor;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.server.HandshakeInterceptor;

import java.util.Map;

import static com.m2pool.chat.constant.Destination.VISITOR_PREFIX;


/**
 * @ClassName WebsocketHandshakeInterceptor
 * @Description websocket 握手处理器类 适用于前端原生websocket实现.建立websocket链接时调用
 * @Author yyb
 * @Date 2025/4/10 15:39
 */
public class WebsocketHandshakeInterceptor implements HandshakeInterceptor {

    private static final Logger LOGGER = LoggerFactory.getLogger(WebsocketHandshakeInterceptor.class);




    @Override
    public boolean beforeHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler, Map<String, Object> attributes) throws Exception {
        LOGGER.info("------------------WebsocketHandshakeInterceptor:beforeHandshake");
//        为游客生成一个唯一标识
//        String s = generateVisitorId(request);
//        attributes.put(VISITOR_PREFIX, s);
//        response.getHeaders().add(VISITOR_PREFIX, s);
        return true;
    }

    @Override
    public void afterHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler, Exception exception) {
        LOGGER.info("------------------WebsocketHandshakeInterceptor:afterHandshake");
    }

    /**
     * 生成一个唯一标识
     * @param request
     * @return
     */
    private String generateVisitorId(ServerHttpRequest request) {
        String first = request.getHeaders().getFirst("sec-websocket-key");
        String prefix = VISITOR_PREFIX;
        return  prefix + first;
    }
}
