package com.m2pool.chat.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.m2pool.chat.dto.ChatMessageDto;
import com.m2pool.chat.entity.ChatMessage;
import com.m2pool.chat.entity.ChatRoom;
import com.m2pool.chat.mapper.ChatMessageHistoryMapper;
import com.m2pool.chat.mapper.ChatMessageMapper;
import com.m2pool.chat.mapper.ChatRoomMapper;
import com.m2pool.chat.service.ChatMessageService;
import com.m2pool.common.core.web.Result.AjaxResult;
import com.m2pool.common.security.utils.SecurityUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

@Service
public class ChatMessageServiceImpl implements ChatMessageService {

    @Resource
    private ChatMessageMapper chatMessageMapper;

    @Resource
    private ChatMessageHistoryMapper chatMessageHistoryMapper;

    @Resource
    private ChatRoomMapper chatRoomMapper;

    @Override
    public AjaxResult findHistoryMessage(Long id,Integer pageNum,Long roomId) {
        List<ChatMessageDto> historyMessage = chatMessageHistoryMapper.findHistoryMessage(id, pageNum, roomId);
        return AjaxResult.success(historyMessage);
    }

    @Override
    public AjaxResult findRecentlyMessage(Long id,Integer pageNum,Long roomId) {
        List<ChatMessageDto> recentlyMessage = chatMessageMapper.findRecentlyMessage(id, pageNum, roomId);
        return AjaxResult.success(recentlyMessage);
    }

    @Override
    public AjaxResult deleteMessage(Long id,Integer type) {
        if(chatMessageMapper.deleteById(id) > 0){
            //TODO 对象图片存储删除
            return AjaxResult.success("删除成功");
        }
        return AjaxResult.error("删除失败");
    }


    @Override
    @Transactional
    public AjaxResult readMessage(Long roomId) {
        ChatRoom chatRoom = chatRoomMapper.selectById(roomId);
        String username = SecurityUtils.getUsername();

        String updateUserEmail = "";
        if(username.equals(chatRoom.getUserOneEmail())){
            updateUserEmail = chatRoom.getUserTwoEmail();
        }else if(username.equals(chatRoom.getUserTwoEmail())){
            updateUserEmail = chatRoom.getUserOneEmail();
        }
        int update = chatMessageMapper.update(ChatMessage.builder().read(1).build(),
                new LambdaUpdateWrapper<ChatMessage>().eq(ChatMessage::getSendEmail, updateUserEmail).eq(ChatMessage::getRoomId, roomId));
        return update > 0 ?  AjaxResult.success("已读") : AjaxResult.error("消息读取失败");
    }
}