package com.m2pool.chat.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageInfo;
import com.m2pool.chat.dto.ChatRoomDto;
import com.m2pool.chat.entity.ChatRoom;
import com.m2pool.chat.mapper.ChatMessageMapper;
import com.m2pool.chat.mapper.ChatRoomMapper;
import com.m2pool.chat.service.ChatRoomService;
import com.m2pool.chat.vo.CharRoomVo;
import com.m2pool.common.core.constant.HttpStatus;
import com.m2pool.common.core.utils.PageUtils;
import com.m2pool.common.core.utils.StringUtils;
import com.m2pool.common.core.web.Result.AjaxResult;
import com.m2pool.common.core.web.page.TableDataInfo;
import com.m2pool.common.security.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ChatRoomServiceImpl extends ServiceImpl<ChatRoomMapper, ChatRoom> implements ChatRoomService {

    @Autowired
    private ChatRoomMapper chatRoomMapper;

    @Resource
    private ChatMessageMapper chatMessageMapper;

    @Override
    public TableDataInfo findRoomList() {
        PageUtils.startPage();
        //1.查找当前客服对应的聊天室
        List<ChatRoomDto> roomList = chatRoomMapper.findRoomList(SecurityUtils.getUsername());
        List<String> userEmails = roomList.stream().map(ChatRoomDto::getUserEmail).collect(Collectors.toList());
        //2.查询未读数量
        Map<String, Integer> unReadNums = chatMessageMapper.findUnReadNums(userEmails);

        for (ChatRoomDto room : roomList) {
            Integer i = unReadNums.get(room.getUserEmail());
            room.setUnReadNumber(i == null ? 0 : i);
        }
        return getDataTable(roomList);
    }

    private TableDataInfo getDataTable(List<?> list)
    {
        TableDataInfo rspData = new TableDataInfo();
        rspData.setCode(HttpStatus.SUCCESS);
        rspData.setRows(list);
        rspData.setMsg("查询成功");
        PageInfo pageInfo = new PageInfo(list);
        rspData.setTotal(pageInfo.getTotal());
        rspData.setTotalPage(pageInfo.getPages());
        return rspData;
    }


    @Override
    @Transactional
    public AjaxResult findRoomByUserid(String email) {
        //1.查询当前用户与对应用户是否已存在创建的聊天室
        String userEmail = SecurityUtils.getUsername();
        String roomByUserEmail = chatRoomMapper.findRoomByUserEmail(userEmail, email);
        if(StringUtils.isNotEmpty(roomByUserEmail)){
            return AjaxResult.success(roomByUserEmail);
        }
        //2.不存在创建一个聊天室
        int insert = chatRoomMapper.insert(ChatRoom.builder().userOneEmail(userEmail).userTwoEmail(email).build());
        if (insert > 0){
            return AjaxResult.success(roomByUserEmail);
        }
        return AjaxResult.error("聊天室不存在,并且创建聊天室失败");
    }

    @Override
    public AjaxResult updateRoom(CharRoomVo charRoomVo) {
        int i = chatRoomMapper.updateById(ChatRoom.builder().id(charRoomVo.getId()).flag(charRoomVo.getFlag()).build());
        if(i > 0){
            return AjaxResult.success("修改成功");
        }
        return AjaxResult.error("修改失败");
    }
}