package com.m2pool.chat.service.impl;

import cn.hutool.core.date.DateTime;
import com.m2pool.chat.constant.Destination;
import com.m2pool.chat.dto.WebsocketMessageDto;
import com.m2pool.chat.entity.ChatMessage;
import com.m2pool.chat.entity.ChatRoom;
import com.m2pool.chat.mapper.ChatMessageMapper;
import com.m2pool.chat.mapper.ChatRoomMapper;
import com.m2pool.chat.service.StompService;
import com.m2pool.chat.vo.UserMessageVo;
import com.m2pool.common.core.web.Result.AjaxResult;
import com.m2pool.common.security.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.messaging.simp.user.SimpUserRegistry;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import static com.m2pool.chat.constant.UserType.CUSTOMER;
import static com.m2pool.chat.constant.UserType.TOURIST;

@Service
public class StompServiceImpl implements StompService {

    @Autowired
    private SimpUserRegistry userRegistry;

    @Autowired
    private SimpMessagingTemplate messagingTemplate;

    @Resource
    private ChatMessageMapper chatMessageMapper;

    @Resource
    private ChatRoomMapper chatRoomMapper;

    @Override
    public AjaxResult sendMessageToUser(UserMessageVo userMessageVo) {
        WebsocketMessageDto websocketMessageDto = new WebsocketMessageDto();
        websocketMessageDto.setType(userMessageVo.getType());
        websocketMessageDto.setContent(userMessageVo.getContent());
        // 检查目标用户是否在线
        if (!checkOnline(userMessageVo)) {
            return AjaxResult.error("目标用户不在线");
        }
        //当前用户发送其他人, 发送给指定用户. 否则默认发送给当前发送者
        messagingTemplate.convertAndSendToUser(userMessageVo.getEmail(), Destination.QUEUE + "/" + userMessageVo.getEmail(), userMessageVo.getContent());
        // 接收者和发送者 都不是游客才能存储消息 和修改聊天室信息
        if (!TOURIST.equals(userMessageVo.getReceiveUserType()) && TOURIST.equals(userMessageVo.getSendUserType())){
            Long id = insertMessage(userMessageVo);
            websocketMessageDto.setId(id);
            updateRoom(userMessageVo);
        }
        return AjaxResult.success(websocketMessageDto);
    }

    /**
     * 检查用户是否在线
     * @return
     */
    private boolean checkOnline(UserMessageVo userMessageVo){
        return userRegistry.getUsers().stream()
                .anyMatch(user -> user.getName().equals(userMessageVo.getEmail()));
    }

    /**
     * 新增消息
     * @param userMessageVo
     * @return
     */
    private Long insertMessage(UserMessageVo userMessageVo){
        ChatMessage message = ChatMessage.builder()
                .sendEmail(SecurityUtils.getUsername())
                .content(userMessageVo.getContent())
                .type(userMessageVo.getType())
                .roomId(userMessageVo.getRoomId())
                .build();
        chatMessageMapper.insert(message);
        return message.getId();
    }

    /**
     * 修改聊天信息
     * @param userMessageVo
     */
    private void updateRoom(UserMessageVo userMessageVo){
        ChatRoom room = ChatRoom.builder().id(userMessageVo.getRoomId()).build();
        if (CUSTOMER.equals(userMessageVo.getSendUserType())) {
            room.setLastCustomerSendTime(DateTime.now());
        } else {
            room.setLastUserSendTime(DateTime.now());
        }
        chatRoomMapper.insert(room);
    }
}